@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 import gtk
 
-from kiwi.ui.widgets.list import List, Column
+from kiwi.ui.objectlist import ObjectList, Column
 
 class NewsItem:
     """An instance that holds information about a news article."""
@@ -9,7 +9,7 @@ class NewsItem:
         self.title, self.author, self.url = title, author, url
 
 # Assemble friendly Pigdog.org news into NewsItem instances so they can
-# be used in the CListDelegate
+# be used in the ObjectList
 news = [
  NewsItem("Smallpox Vaccinations for EVERYONE", "JRoyale",
           "http://www.pigdog.org/auto/Power_Corrupts/link/2700.html"),
@@ -26,18 +26,18 @@ news = [
 
 # Specify the columns: one for each attribute of NewsItem, the URL
 # column invisible by default
-my_columns = [Column("title", sorted=True), 
-              Column("author", justify=gtk.JUSTIFY_RIGHT), 
+my_columns = [Column("title", sorted=True),
+              Column("author", justify=gtk.JUSTIFY_RIGHT),
               Column("url", title="URL", visible=False)]
 
-kiwilist = List(my_columns)
-kiwilist.add_list(news)
+objectlist = ObjectList(my_columns)
+objectlist.add_list(news)
 
 w = gtk.Window()
 w.connect('delete-event', gtk.main_quit)
 w.set_size_request(600, 250)
 
-w.add(kiwilist)
+w.add(objectlist)
 
 w.show_all()
 gtk.main()
