@@ -56,6 +56,10 @@ class ConverterRegistry:
         self._converters = {}
 
     def add(self, converter_type):
+        """
+        Adds converter_type as a new converter
+        @param converter_type: a L{BaseConverter} subclass
+        """
         if not issubclass(converter_type, BaseConverter):
             raise TypeError("converter_type must be a BaseConverter subclass")
 
@@ -234,9 +238,33 @@ class _FloatConverter(BaseConverter):
 
     def as_string(self, value, format=None):
         """Convert a float to a string"""
+        format_set = True
+
         if format is None:
-            return str(float(value))
-        return lformat(format, value)
+            # From Objects/floatobject.c:
+            #
+            # The precision (12) is chosen so that in most cases, the rounding noise
+            # created by various operations is suppressed, while giving plenty of
+            # precision for practical use.
+            format = '%.12g'
+
+            format_set = False
+
+        as_str = lformat(format, value)
+
+        # If the format was not set, the resoult should be treated, as
+        # follows.
+        if not format_set and not value % 1:
+            # value % 1 is used to check if value has an decimal part. If it
+            # doen't then it's an integer
+
+            # When format is '%g', if value is an integer, the result
+            # will also be formated as an integer, so we add a '.0'
+
+            conv = locale.localeconv()
+            as_str +=  conv.get('decimal_point') + '0'
+
+        return as_str
 
     def from_string(self, value):
         """Convert a string to a float"""
@@ -626,7 +654,3 @@ def filter_locale(value):
     if decimal_point != '.':
         value = value.replace(decimal_point, '.')
     return value
-
-# by default locale uses the C locale but our date conversions use the user
-# locale so we need to set the locale to that one
-locale.setlocale(locale.LC_ALL, '') # this set the user locale ( $LANG )
