@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2005 Async Open Source
+# Copyright (C) 2005-2006 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -25,14 +25,18 @@
 
 import gettext
 import imp
+import locale
 import os
 import sys
 
+from kiwi.log import Logger
 from kiwi.python import namedAny
 
 __all__ = ['Application', 'Library', 'app', 'environ', 'require_gazpacho',
            'is_gazpacho_required']
 
+log = Logger('environ')
+
 EnvironmentError = EnvironmentError
 
 # From http://tinyurl.com/77ukj
@@ -52,8 +56,14 @@ class Environment:
 
     def __init__(self, root='.'):
         self._resources = {}
+        self._extensions = {}
         self._root = root
 
+        # Add some compressed formats as alternative extensions to
+        # "glade" resources. A patch has been added to gazpacho trunk
+        # (rev. 2251) to support loading those compressed formats.
+        self._add_extensions("glade", ".bz2", ".gz")
+
         self._add_resource_variable("glade", "KIWI_GLADE_PATH")
         self._add_resource_variable("image", "KIWI_IMAGE_PATH")
 
@@ -63,6 +73,10 @@ class Environment:
     def get_log_level(self):
         return os.environ.get('KIWI_LOG')
 
+    def _add_extensions(self, resource, *args):
+        exts = self._extensions.setdefault(resource, [])
+        exts.extend(list(args))
+
     def _add_resource_variable(self, resource, variable):
         """Add resources from an environment variable"""
         env = os.environ.get(variable, '')
@@ -92,15 +106,19 @@ class Environment:
         """Locate a specific resource of called name of type resource"""
 
         resource_paths = self.get_resource_paths(resource)
-        for path in resource_paths:
-            filename = os.path.join(self._root, path, name)
-            if os.path.exists(filename):
-                return filename
 
-        # Finally try to load the file from the current directory
-        filename = os.path.join(self._root, name)
-        if os.path.exists(filename):
-            return filename
+        # Look for alternative extensions for this resource.
+        # But check without extensions first
+        exts = [""] + self._extensions.get(resource, [])
+
+        # Check "scriptdir", which is the directory the script is ran from
+        # and the working directory ("") after all the others fail
+        scriptdir = os.path.dirname(os.path.abspath(sys.argv[0]))
+        for path in resource_paths + [scriptdir, ""]:
+            for ext in exts:
+                filename = os.path.join(self._root, path, "".join((name, ext)))
+                if os.path.exists(filename):
+                    return filename
 
         raise EnvironmentError("Could not find %s resource: %s" % (
             resource, name))
@@ -174,6 +192,7 @@ class Library(Environment):
 
         # py2exe
         if _is_frozen():
+            log.info('py2exe found')
             executable = os.path.realpath(os.path.abspath(sys.executable))
             root = os.path.dirname(executable)
         # normal
@@ -214,17 +233,61 @@ class Library(Environment):
 
         self.uninstalled = uninstalled
 
-    def enable_translation(self, domain=None, locale='locale',
-                           charset='utf-8'):
+    def _check_translation(self, domain, directory):
+        loc = locale.getlocale()[0]
+
+        # We're not interested in warnings for these locales
+        if loc in (None, 'C', 'en_US'):
+            return
+
+        # check sv_SE and sv
+        locales = [loc]
+        if '_' in loc:
+            locales.append(loc.split('_')[0])
+
+        for l in locales:
+            path = os.path.join(directory, l, 'LC_MESSAGES', domain + '.mo')
+            if os.path.exists(path):
+                break
+        else:
+            log.warn('No %s translation found for domain %s' % (loc, domain))
+
+    def enable_translation(self, domain=None, localedir=None):
+        """
+        Enables translation for a library
+
+        @param domain: optional, if not specified name sent to constructor
+          will be used
+        @param localedir: directory to get locales from when running in
+          uninstalled mode. If not specified a directory called 'locale' in
+          the root will be used.
+        """
         if not domain:
             domain = self.name
+
+        if not localedir:
+            localedir = 'locale'
+
+        if self.uninstalled:
+            try:
+                self.add_resource('locale', localedir)
+            except EnvironmentError:
+                pass
+
         # XXX: locale should not be a list
-        localedir = self._resources.get(locale)
-        if localedir:
-            gettext.bindtextdomain(domain, localedir[0])
-        else:
-            print 'Warning, no localedir for: %s' % domain
-        gettext.bind_textdomain_codeset(domain, charset)
+        localedir = self._resources.get('locale')
+        if not localedir:
+            # Only complain when running installed
+            if not self.uninstalled:
+                log.warn('no localedir for: %s' % domain)
+
+            return
+        directory = gettext.bindtextdomain(domain, localedir[0])
+        self._check_translation(domain, directory)
+
+        # Gtk+ only supports utf-8, it makes no sense to support
+        # other encodings in kiwi it self
+        gettext.bind_textdomain_codeset(domain, 'utf-8')
 
     def set_application_domain(self, domain):
         """
@@ -279,9 +342,9 @@ class Application(Library):
     def _get_main(self):
         try:
             module = namedAny(self._path)
-        except ImportError, e:
-            raise SystemExit("ERROR: Could not find item '%s', %s" %
-                             (self._path, e))
+        except:
+            log.warn('importing %s' % self._path)
+            raise
 
         main = getattr(module, 'main', None)
         if not main or not callable(main):
@@ -289,15 +352,16 @@ class Application(Library):
                              'main', self._path)
         return main
 
-    def enable_translation(self, domain=None, locale='locale',
-                           charset='utf-8'):
-        if not domain:
-            domain = self.name
+    def enable_translation(self, domain=None, localedir=None):
+        """
+        Enables translation for a application
+        See L{Library.enable_translation}.
 
-        Library.enable_translation(self, domain, locale, charset)
+        """
+        Library.enable_translation(self, domain, localedir)
         old_domain = gettext.textdomain()
         if old_domain  != 'messages':
-            print 'Warning: overriding default domain, was %s' % old_domain
+            log.warn('overriding default domain, was %s' % old_domain)
 
         self.set_application_domain(domain)
 
