@@ -26,8 +26,8 @@
 import gtk
 from gtk import gdk, keysyms
 
+from kiwi.ui.entry import KiwiEntry
 from kiwi.utils import gsignal, type_register
-from kiwi.ui.widgets.entry import Entry
 
 class _ComboEntryPopup(gtk.Window):
     gsignal('text-selected', str)
@@ -76,7 +76,6 @@ class _ComboEntryPopup(gtk.Window):
 
         self._label = gtk.Label()
         vbox.pack_start(self._label, False, False)
-        self._label.show()
 
         self.set_resizable(False)
         self.set_screen(comboentry.get_screen())
@@ -91,16 +90,18 @@ class _ComboEntryPopup(gtk.Window):
             return
 
         treeview = self._treeview
-        if treeview.flags() & gtk.MAPPED:
-            return
         toplevel = combo.get_toplevel()
         if isinstance(toplevel, gtk.Window) and toplevel.group:
             toplevel.group.add_window(self)
 
+        # width is meant for the popup window
+        # height is meant for the treeview, since it calculates using
+        # the height of the cells on the rows
         x, y, width, height = self._get_position()
-        self.set_size_request(width, height)
+        self.set_size_request(width, -1)
+        treeview.set_size_request(-1, height)
         self.move(x, y)
-        self.show_all()
+        self.show()
 
         treeview.set_hover_expand(True)
         selection = treeview.get_selection()
@@ -129,9 +130,14 @@ class _ComboEntryPopup(gtk.Window):
             return
 
         self.grab_remove()
-        self.hide_all()
+        self.hide()
 
     def set_label_text(self, text):
+        if text is None:
+            text = ''
+            self._label.hide()
+        else:
+            self._label.show()
         self._label.set_text(text)
 
     def set_model(self, model):
@@ -215,7 +221,6 @@ class _ComboEntryPopup(gtk.Window):
     def _get_position(self):
         treeview = self._treeview
         treeview.realize()
-        cell = treeview.get_cell_area(0, treeview.get_column(0))
 
         sample = self._comboentry
 
@@ -237,11 +242,9 @@ class _ComboEntryPopup(gtk.Window):
             rows = self._visible_rows
             self._sw.set_policy(hpolicy, gtk.POLICY_ALWAYS)
 
-        # This needs some quite serious improvements
         focus_padding = treeview.style_get_property('focus-line-width') * 2
-        height = (cell.height *
-                  (rows + focus_padding) -
-                  (2 + focus_padding))
+        cell_height = treeview.get_column(0).cell_get_size()[4]
+        height = (cell_height + focus_padding) * rows
 
         screen = self._comboentry.get_screen()
         monitor_num = screen.get_monitor_at_window(sample.window)
@@ -283,13 +286,21 @@ type_register(_ComboEntryPopup)
 class ComboEntry(gtk.HBox):
     gsignal('changed')
     gsignal('activate')
-    def __init__(self):
+    def __init__(self, entry=None):
+        """
+        @param entry: a gtk.Entry subclass to use
+        """
         gtk.HBox.__init__(self)
         self._popping_down = False
 
-        self.entry = Entry()
+        if not entry:
+            entry = KiwiEntry()
+
+        self.entry = entry
         self.entry.connect('activate',
                            self._on_entry__activate)
+        self.entry.connect('changed',
+                           self._on_entry__changed)
         self.entry.connect('scroll-event',
                            self._on_entry__scroll_event)
         self.entry.connect('key-press-event',
@@ -313,7 +324,9 @@ class ComboEntry(gtk.HBox):
         self._popup.connect('hide', self._on_popup__hide)
         self._popup.set_size_request(-1, 24)
 
-        self.set_model(self.entry.get_completion().get_model())
+        completion = gtk.EntryCompletion()
+        self.entry.set_completion(completion)
+        self.set_model(completion.get_model())
 
     # Virtual methods
 
@@ -322,9 +335,17 @@ class ComboEntry(gtk.HBox):
 
     # Callbacks
 
+    def _on_entry_completion__match_selected(self, completion, model, iter):
+        # the iter we receive is specific to the tree model filter used
+        # In the entry completion, convert it to an iter in the real model
+        self.set_active_iter(model.convert_iter_to_child_iter(iter))
+
     def _on_entry__activate(self, entry):
         self.emit('activate')
 
+    def _on_entry__changed(self, entry):
+        self.emit('changed')
+
     def _on_entry__scroll_event(self, entry, event):
         model = self.get_model()
         treeiter = self._popup.get_selected_iter()
@@ -411,7 +432,6 @@ class ComboEntry(gtk.HBox):
         @param text:
         """
         self.entry.set_text(text)
-        self.emit('changed')
 
     def get_text(self):
         """
@@ -430,6 +450,8 @@ class ComboEntry(gtk.HBox):
         self._model = model
         self._popup.set_model(model)
         completion = self.entry.get_completion()
+        completion.connect('match-selected',
+                           self._on_entry_completion__match_selected)
         completion.set_model(model)
 
         self._update()
@@ -501,6 +523,9 @@ class ComboEntry(gtk.HBox):
         treeiter = self.entry.get_iter_from_obj(obj)
         self.set_active_iter(treeiter)
 
+    def set_label_text(self, text):
+        self._popup.set_label_text(text)
+
     # IconEntry
 
     def set_pixbuf(self, pixbuf):
