@@ -25,8 +25,10 @@
 """Gazpacho integration: loader and extensions"""
 
 import datetime
+from decimal import Decimal
 import gettext
 import os
+import warnings
 
 import gobject
 import gtk
@@ -35,25 +37,32 @@ from gazpacho.loader.loader import ObjectBuilder
 from gazpacho.loader.custom import Adapter, ComboBoxAdapter, \
      PythonWidgetAdapter, adapter_registry
 from gazpacho.properties import prop_registry, CustomProperty, StringType
+from gazpacho.widgets.base.base import ContainerAdaptor
+from gazpacho.widgets.base.box import BoxAdaptor
 
-from kiwi import _warn
 from kiwi.datatypes import currency
 from kiwi.environ import environ
+from kiwi.log import Logger
+from kiwi.python import disabledeprecationcall
+from kiwi.ui.hyperlink import HyperLink
 from kiwi.ui.objectlist import Column, ObjectList
-from kiwi.ui.widgets.checkbutton import CheckButton
-from kiwi.ui.widgets.combo import ProxyComboEntry, ProxyComboBox, ProxyComboBoxEntry
-from kiwi.ui.widgets.entry import Entry
-from kiwi.ui.widgets.label import Label
-from kiwi.ui.widgets.radiobutton import RadioButton
-from kiwi.ui.widgets.spinbutton import SpinButton
-from kiwi.ui.widgets.textview import TextView
+from kiwi.ui.widgets.checkbutton import ProxyCheckButton
+from kiwi.ui.widgets.combo import ProxyComboEntry, ProxyComboBox, \
+     ProxyComboBoxEntry
+from kiwi.ui.widgets.entry import ProxyDateEntry, ProxyEntry
+from kiwi.ui.widgets.label import ProxyLabel
+from kiwi.ui.widgets.radiobutton import ProxyRadioButton
+from kiwi.ui.widgets.spinbutton import ProxySpinButton
+from kiwi.ui.widgets.textview import ProxyTextView
 
 # Backwards compatibility + pyflakes
 from kiwi.ui.widgets.combobox import ComboBox, ComboBoxEntry
 from kiwi.ui.widgets.list import List
-
+HyperLink
 _ = gettext.gettext
 
+log = Logger('gazpacholoader')
+
 class Builder(ObjectBuilder):
     def find_resource(self, filename):
         return environ.find_resource("pixmaps",  filename)
@@ -70,17 +79,24 @@ class GazpachoWidgetTree:
             raise TypeError(
                   "gladefile should be a string, found %s" % type(gladefile))
         filename = os.path.splitext(os.path.basename(gladefile))[0]
-
+        self._filename = filename + '.glade'
         self._view = view
-        self._gladefile = environ.find_resource("glade", filename + ".glade")
+        self._gladefile = environ.find_resource("glade", self._filename)
         self._widgets =  (widgets or view.widgets or [])[:]
         self.gladename = gladename or filename
+        self._showwarning = warnings.showwarning
+        warnings.showwarning = self._on_load_warning
         self._tree = Builder(self._gladefile, domain=domain)
+        warnings.showwarning = self._showwarning
         if not self._widgets:
             self._widgets = [w.get_data("gazpacho::object-id")
                              for w in self._tree.get_widgets()]
         self._attach_widgets()
 
+    def _on_load_warning(self, warning, category, file, line):
+        self._showwarning('while loading glade file: %s' % warning,
+                          category, self._filename, '???')
+
     def _attach_widgets(self):
         # Attach widgets in the widgetlist to the view specified, so
         # widgets = [label1, button1] -> view.label1, view.button1
@@ -89,7 +105,7 @@ class GazpachoWidgetTree:
             if widget is not None:
                 setattr(self._view, w, widget)
             else:
-                _warn("Widget %s was not found in glade widget tree." % w)
+                log.warn("Widget %s was not found in glade widget tree." % w)
 
     def get_widget(self, name):
         """Retrieves the named widget from the View (or glade tree)"""
@@ -107,6 +123,47 @@ class GazpachoWidgetTree:
     def signal_autoconnect(self, dic):
         self._tree.signal_autoconnect(dic)
 
+    def get_sizegroups(self):
+        return self._tree.sizegroups
+
+# Normal widgets
+for prop in ('normal-color', 'normal-underline', 'normal-bold',
+             'hover-color', 'hover-underline', 'hover-bold',
+             'active-color', 'active-underline', 'active-bold'):
+    prop_registry.override_simple('HyperLink::%s' % prop, editable=False)
+
+class HyperLinkAdaptor(ContainerAdaptor):
+    def fill_empty(self, context, widget):
+        pass
+
+    def post_create(self, context, widget, interactive):
+        widget.set_text(widget.get_name())
+
+class ComboEntryAdaptor(BoxAdaptor):
+    def get_children(self, context, comboentry):
+        return []
+
+class DateEntryAdaptor(BoxAdaptor):
+    def get_children(self, context, comboentry):
+        return []
+
+class KiwiColumnAdapter(Adapter):
+    object_type = Column
+    def construct(self, name, gtype, properties):
+        return Column(name)
+adapter_registry.register_adapter(KiwiColumnAdapter)
+
+class ObjectListAdapter(PythonWidgetAdapter):
+    object_type = ObjectList
+    def construct(self, name, gtype, properties):
+        if gtype == List:
+            gtype == ObjectList
+        return super(ObjectListAdapter, self).construct(name, gtype,
+                                                        properties)
+adapter_registry.register_adapter(ObjectListAdapter)
+
+# Framework widgets
+
 class DataTypeAdaptor(PropertyCustomEditor):
     def __init__(self):
         super(DataTypeAdaptor, self).__init__()
@@ -159,6 +216,7 @@ class SpinBtnDataType(DataTypeAdaptor):
         return [
             (_('Integer'), int),
             (_('Float'), float),
+            (_('Decimal'), Decimal),
             (_('Currency'), currency)
             ]
 
@@ -169,9 +227,10 @@ class EntryDataType(DataTypeAdaptor):
             (_('Unicode'), unicode),
             (_('Integer'), int),
             (_('Float'), float),
+            (_('Decimal'), Decimal),
             (_('Currency'), currency),
             (_('Date'), datetime.date),
-            (_('Datetime'), datetime.datetime),
+            (_('Date and Time'), datetime.datetime),
             (_('Time'), datetime.time),
             (_('Object'), object)
             ]
@@ -183,6 +242,7 @@ class TextViewDataType(DataTypeAdaptor):
             (_('Unicode'), unicode),
             (_('Integer'), int),
             (_('Float'), float),
+            (_('Decimal'), Decimal),
             (_('Date'), datetime.date),
             ]
 
@@ -194,6 +254,7 @@ class ComboBoxDataType(DataTypeAdaptor):
             (_('Boolean'), bool),
             (_('Integer'), int),
             (_('Float'), float),
+            (_('Decimal'), Decimal),
             (_('Object'), object)
             ]
 
@@ -205,42 +266,37 @@ class LabelDataType(DataTypeAdaptor):
             (_('Boolean'), bool),
             (_('Integer'), int),
             (_('Float'), float),
+            (_('Decimal'), Decimal),
             (_('Date'), datetime.date),
-            (_('Datetime'), datetime.datetime),
+            (_('Date and Time'), datetime.datetime),
             (_('Time'), datetime.time),
             (_('Currency'), currency)
             ]
 
-class DataTypeProperty(CustomProperty, StringType):
+class DataType(CustomProperty, StringType):
     translatable = False
     def save(self):
         value = self.get()
         if value is not None:
             return value.__name__
 
-class BoolDataTypeProperty(CustomProperty, StringType):
+class BoolOnlyDataType(CustomProperty, StringType):
     translatable = False
     editable = False
     def save(self):
         return 'bool'
 
-class ModelProperty(CustomProperty):
+class DateOnlyDataType(CustomProperty, StringType):
+    translatable = False
+    editable = False
+    def save(self):
+        return 'date'
+
+class ModelProperty(CustomProperty, StringType):
     translatable = False
 
-class KiwiColumnAdapter(Adapter):
-    object_type = Column
-    def construct(self, name, gtype, properties):
-        return Column(name)
-adapter_registry.register_adapter(KiwiColumnAdapter)
-
-class ObjectListAdapter(PythonWidgetAdapter):
-    object_type = ObjectList
-    def construct(self, name, gtype, properties):
-        if gtype == List:
-            gtype == ObjectList
-        return super(ObjectListAdapter, self).construct(name, gtype,
-                                                        properties)
-adapter_registry.register_adapter(ObjectListAdapter)
+class DataValueProperty(CustomProperty, StringType):
+    translatable = False
 
 # Register widgets which have data-type and model-attributes
 # ComboBox is a special case, it needs to inherit from another
@@ -257,46 +313,48 @@ class KiwiComboBoxAdapter(ComboBoxAdapter):
         else:
             raise AssertionError("Unknown ComboBox GType: %r" % gtype)
 
-        obj = object_type()
+        obj = disabledeprecationcall(object_type)
         obj.set_name(name)
         return obj
 adapter_registry.register_adapter(KiwiComboBoxAdapter)
 
-for gobj, editor in [(Entry, EntryDataType),
-                     (CheckButton, None),
-                     (Label, LabelDataType),
-                     (ProxyComboBox, ComboBoxDataType),
-                     (ProxyComboBoxEntry, ComboBoxDataType),
-                     (ProxyComboEntry, ComboBoxDataType),
-                     (SpinButton, SpinBtnDataType),
-                     (RadioButton, None),
-                     (TextView, TextViewDataType)]:
-    # Property overrides, used in the editor
-    type_name = gobject.type_name(gobj)
+def register_widgets():
+    for gobj, editor, data_type in [
+        (ProxyEntry, EntryDataType, DataType),
+        (ProxyDateEntry, None, DateOnlyDataType),
+        (ProxyCheckButton, None, BoolOnlyDataType),
+        (ProxyLabel, LabelDataType, DataType),
+        (ProxyComboBox, ComboBoxDataType, DataType),
+        (ProxyComboBoxEntry, ComboBoxDataType, DataType),
+        (ProxyComboEntry, ComboBoxDataType, DataType),
+        (ProxySpinButton, SpinBtnDataType, DataType),
+        (ProxyRadioButton, None, BoolOnlyDataType),
+        (ProxyTextView, TextViewDataType, DataType)
+        ]:
+        # Property overrides, used in the editor
+        type_name = gobject.type_name(gobj)
 
-    # This is a hack for epydoc
-    if type_name is None:
-        import sys
-        assert sys.argv == ['IGNORE']
-        continue
+        data_name = type_name + '::data-type'
+        if editor:
+            prop_registry.override_simple(data_name, data_type, editor=editor)
+        else:
+            prop_registry.override_simple(data_name, data_type)
 
-    data_name = type_name + '::data-type'
-    if editor:
-        prop_registry.override_simple(data_name, DataTypeProperty,
-                                      editor=editor)
-    else:
-        prop_registry.override_simple(data_name, BoolDataTypeProperty)
+        prop_registry.override_simple(type_name + '::model-attribute',
+                                      ModelProperty)
 
+        if issubclass(gobj, ProxyRadioButton):
+            prop_registry.override_simple(type_name + '::data-value',
+                                          DataValueProperty)
+        # Register custom adapters, since gobject.new is broken in 2.6
+        # Used by loader, eg in gazpacho and in applications
+        # ComboBox is registered above
+        if gobj == ProxyComboBox:
+            continue
 
-    prop_registry.override_simple(type_name + '::model-attribute',
-                                  ModelProperty)
+        klass = type('Kiwi%sAdapter', (PythonWidgetAdapter,),
+                     dict(object_type=gobj))
+        adapter_registry.register_adapter(klass)
 
-    # Register custom adapters, since gobject.new is broken in 2.6
-    # Used by loader, eg in gazpacho and in applications
-    # ComboBox is registered above
-    if gobj == ProxyComboBox:
-        continue
-
-    klass = type('Kiwi%sAdapter', (PythonWidgetAdapter,),
-                 dict(object_type=gobj))
-    adapter_registry.register_adapter(klass)
+if not environ.epydoc:
+    register_widgets()
