@@ -37,21 +37,23 @@ import gtk
 from gtk import keysyms
 
 from kiwi import ValueUnset
+from kiwi.python import deprecationwarn
 from kiwi.ui.comboboxentry import BaseComboBoxEntry
 from kiwi.ui.comboentry import ComboEntry
 from kiwi.ui.combomixin import COL_COMBO_LABEL, COMBO_MODE_STRING, \
      COMBO_MODE_DATA, COMBO_MODE_UNKNOWN, ComboMixin
-from kiwi.ui.widgets.proxy import WidgetMixin, WidgetMixinSupportValidation
+from kiwi.ui.proxywidget import ProxyWidgetMixin, ValidatableProxyWidgetMixin
+from kiwi.ui.widgets.entry import ProxyEntry
 from kiwi.utils import PropertyObject, gproperty
 
-class ProxyComboBox(PropertyObject, gtk.ComboBox, ComboMixin, WidgetMixin):
+class ProxyComboBox(PropertyObject, gtk.ComboBox, ComboMixin, ProxyWidgetMixin):
 
     __gtype_name__ = 'ProxyComboBox'
 
     def __init__(self):
         gtk.ComboBox.__init__(self)
         ComboMixin.__init__(self)
-        WidgetMixin.__init__(self)
+        ProxyWidgetMixin.__init__(self)
         PropertyObject.__init__(self)
         self.connect('changed', self._on__changed)
 
@@ -103,7 +105,7 @@ class ProxyComboBox(PropertyObject, gtk.ComboBox, ComboMixin, WidgetMixin):
         self.emit('content-changed')
 
 class ProxyComboBoxEntry(PropertyObject, BaseComboBoxEntry, ComboMixin,
-                         WidgetMixinSupportValidation):
+                         ValidatableProxyWidgetMixin):
     __gtype_name__ = 'ProxyComboBoxEntry'
     # it doesn't make sense to connect to this signal
     # because we want to monitor the entry of the combo
@@ -112,9 +114,12 @@ class ProxyComboBoxEntry(PropertyObject, BaseComboBoxEntry, ComboMixin,
     gproperty("list-editable", bool, True, "Editable")
 
     def __init__(self, **kwargs):
+        deprecationwarn(
+            'ProxyComboBoxEntry is deprecated, use ProxyComboEntry instead',
+            stacklevel=3)
         BaseComboBoxEntry.__init__(self)
         ComboMixin.__init__(self)
-        WidgetMixinSupportValidation.__init__(self, widget=self.entry)
+        ValidatableProxyWidgetMixin.__init__(self, widget=self.entry)
         PropertyObject.__init__(self, **kwargs)
 
         self.set_text_column(COL_COMBO_LABEL)
@@ -209,35 +214,28 @@ class ProxyComboBoxEntry(PropertyObject, BaseComboBoxEntry, ComboMixin,
         self.entry.set_text("")
 
 class ProxyComboEntry(PropertyObject, ComboEntry,
-                      WidgetMixinSupportValidation):
+                      ValidatableProxyWidgetMixin):
     __gtype_name__ = 'ProxyComboEntry'
 
     gproperty("list-editable", bool, True, "Editable")
 
     def __init__(self):
-        self.mode = COMBO_MODE_STRING
-        ComboEntry.__init__(self)
-        WidgetMixinSupportValidation.__init__(self)
+        entry = ProxyEntry()
+        ComboEntry.__init__(self, entry=entry)
+        ValidatableProxyWidgetMixin.__init__(self)
         PropertyObject.__init__(self)
-        self.connect('changed', self._on__changed)
-
-    def _on__changed(self, widget):
-        if not self.entry.get_text():
-            return
+        entry.connect('content-changed', self._on_entry__content_changed)
 
+    # We only need to listen for changes in the entry, it's updated
+    # even if you select something in the popup list
+    def _on_entry__content_changed(self, entry):
         self.emit('content-changed')
 
     def prop_set_list_editable(self, value):
-        if self.mode == COMBO_MODE_DATA:
-            raise AssertionError("You cannot use list-editable in data mode")
-
         self.entry.set_editable(value)
-
         return value
 
     def read(self):
-        if self.mode == COMBO_MODE_UNKNOWN:
-            return ValueUnset
         return self.get_selected()
 
     def update(self, data):
@@ -250,3 +248,7 @@ class ProxyComboEntry(PropertyObject, ComboEntry,
         """Removes all items from list and erases entry"""
         ComboMixin.clear(self)
         self.entry.set_text("")
+
+    def set_tooltip(self, text):
+        self.entry.set_tooltip(text)
+
