@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2003-2005 Async Open Source
+# Copyright (C) 2003-2006 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -29,15 +29,19 @@
 import gtk
 
 from kiwi import ValueUnset
-from kiwi.utils import PropertyObject, gproperty
-from kiwi.ui.widgets.proxy import WidgetMixin
+from kiwi.python import deprecationwarn
+from kiwi.utils import PropertyObject, gproperty, type_register
+from kiwi.ui.proxywidget import ProxyWidgetMixin
 
-class RadioButton(PropertyObject, gtk.RadioButton, WidgetMixin):
+class ProxyRadioButton(PropertyObject, gtk.RadioButton, ProxyWidgetMixin):
+    __gtype_name__ = 'ProxyRadioButton'
     gproperty('data-value', str, nick='Data Value')
 
-    def __init__(self):
-        gtk.RadioButton.__init__(self)
-        WidgetMixin.__init__(self)
+    def __init__(self, group=None, label=None, use_underline=True):
+        gtk.RadioButton.__init__(self, None, label, use_underline)
+        if group:
+            self.set_group(group)
+        ProxyWidgetMixin.__init__(self)
         PropertyObject.__init__(self)
         self.connect('group-changed', self._on_group_changed)
 
@@ -69,9 +73,22 @@ class RadioButton(PropertyObject, gtk.RadioButton, WidgetMixin):
 
     def update(self, data):
         if data is None:
+            # In a group of radiobuttons, the only widget which is in
+            # the proxy is ourself, the other buttons do not get their
+            # update() method called, so the default value is activate
+            # ourselves when the model is empty
+            self.set_active(True)
             return
 
         data = self._as_string(data)
         for rb in self.get_group():
             if rb.get_property('data-value') == data:
                 rb.set_active(True)
+
+class RadioButton(ProxyRadioButton):
+    def __init__(self):
+        deprecationwarn(
+            'RadioButton is deprecated, use ProxyRadioButton instead',
+            stacklevel=3)
+        ProxyRadioButton.__init__(self)
+type_register(RadioButton)
